// PWBNav.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "PWBNav.h"
#include "PWBNavDlg.h"
#include <atlbase.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CPWBNavApp

BEGIN_MESSAGE_MAP(CPWBNavApp, CWinApp)
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()


// CPWBNavApp construction

CPWBNavApp::CPWBNavApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
	m_bOpenNew = FALSE;
	m_bRunHidden = FALSE;
	m_bCloseSecondary = FALSE;
	m_bReadINIFile = FALSE;
}


// The one and only CPWBNavApp object

CPWBNavApp theApp;


// CPWBNavApp initialization

BOOL CPWBNavApp::InitInstance()
{
	// InitCommonControls() is required on Windows XP if an application
	// manifest specifies use of ComCtl32.dll version 6 or later to enable
	// visual styles.  Otherwise, any window creation will fail.
	InitCommonControls();

	CWinApp::InitInstance();

	AfxEnableControlContainer();

	// get the command line information
	GetCommandLine();

	BOOL bDone = FALSE;
	// check for INI file
	if( !m_szINI.IsEmpty() )
	{
		// write INI file if specified and exit if Ok
		if( WriteINIFile( m_szINI ))
			bDone = TRUE;
	}
	// check for close secondary
	if( m_bCloseSecondary && m_szURL.IsEmpty() )
	{
		// close windows
		if( CloseSecondary() )
			bDone = TRUE;
	}
	// check for URL
	if( !m_szURL.IsEmpty() )
	{
		// navigate to URL and exit if Ok
		if( NavigateURL( m_szURL, m_bOpenNew, m_szRun ))
			bDone = TRUE;
	}
	// check for run file
	if( !m_szRun.IsEmpty() )
	{
		// run file if specified
		if( StartApplication( m_szRun ))
			bDone = TRUE;
	}	
	// check for read INI file
	if( m_bReadINIFile )
	{
		// run file if specified
		if( ReadINIFile( m_szRun ))
			bDone = TRUE;
	}

	// return FALSE to exit
	if( bDone )
		return FALSE;

	// show error dialog
	CPWBNavDlg dlg;
	m_pMainWnd = &dlg;
	INT_PTR nResponse = dlg.DoModal();
	if (nResponse == IDOK)
	{
		// TODO: Place code here to handle when the dialog is
		//  dismissed with OK
	}
	else if (nResponse == IDCANCEL)
	{
		// TODO: Place code here to handle when the dialog is
		//  dismissed with Cancel
	}

	// Since the dialog has been closed, return FALSE so that we exit the
	//  application, rather than start the application's message pump.
	return FALSE;
}

void CPWBNavApp::GetCommandLine(void)
{
	for( INT i = 1; i < __argc; i++ )
	{
		LPCTSTR pszParam = __targv[i];
		BOOL bFlag = FALSE;
		BOOL bLast = ((i + 1) == __argc);
		if (pszParam[0] == '-' || pszParam[0] == '/')
		{
			// remove flag specifier
			bFlag = TRUE;
			++pszParam;
		}
		if( bFlag )
		{
			CString csFlag = pszParam;
			csFlag.MakeLower();
			if( csFlag.Left( 1 ) == _T("s") )
				m_bCloseSecondary = TRUE;
			if( csFlag.Left( 1 ) == _T("h") )
				m_bRunHidden = TRUE;
			if( csFlag.Left( 1 ) == _T("r") )
				m_szRun = csFlag.Right( csFlag.GetLength() - 2 );
			if( csFlag.Left( 1 ) == _T("n") )
				m_bOpenNew = TRUE;
			if( csFlag.Left( 1 ) == _T("c") )
				m_bReadINIFile = TRUE;
			if( csFlag.Left( 1 ) == _T("w") )
				m_szWord = csFlag.Right( csFlag.GetLength() - 2 );
			if( csFlag.Left( 3 ) == _T("ini") )
				m_szINI = csFlag.Right( csFlag.GetLength() - 4 );
		}
		else		
			m_szURL = pszParam;
	}
}

BOOL CPWBNavApp::WriteINIFile(CString szINI)
{
	BOOL bRetVal = FALSE;
	// use API to write INI file
	bRetVal = WritePrivateProfileString( _T("Status"), _T("Word"), m_szWord, szINI );
	bRetVal = WritePrivateProfileString( _T("Status"), _T("URL"), m_szURL, szINI );
	return bRetVal;
}

BOOL CPWBNavApp::CloseSecondary(void)
{
	// get window handle
	HWND hWnd = ::FindWindow( _T("TSSPWB"), NULL );
	// loop through windows
	if( hWnd )
	{
		COPYDATASTRUCT CDS;
		// set read INI type
		DWORD dwType = WM_USER_CLOSE_SECONDARY;
		// copy data
		CDS.cbData = NULL;
		CDS.dwData = dwType;
		CDS.lpData = NULL;
		// send message to applciation
		::SendMessage( hWnd, WM_COPYDATA, NULL, (LPARAM)(LPVOID) &CDS );
		// wait a second
		Sleep( 1000 );
		return TRUE;
	}
	return FALSE;
}

BOOL CPWBNavApp::NavigateURL(CString szURL, BOOL bOpenNew, CString szApplication)
{
	// get window handle
	HWND hWnd = ::FindWindow( _T("TSSPWB"), NULL );
	if(hWnd == NULL)
		hWnd = ::FindWindow(_T("TSSPWB3"), NULL);
	
	// if it is not running start it
	if( !hWnd )
	{
		// get PWB path
		if( szApplication.IsEmpty() )
		{
			CRegKey key;
			// open the registry location and get path
			if( key.Open( HKEY_CURRENT_USER, _T("Software\\TeamSoftware Solutions\\Public Web Browser v2")) == ERROR_SUCCESS )
			{
				TCHAR getInfoString[500];
				ULONG lengthVal = 500;
				memset( getInfoString, NULL, 500 );
				key.QueryStringValue( _T( "App_Location" ), getInfoString, &lengthVal );
				// set application string
				szApplication = getInfoString;
			}
			else if (key.Open(HKEY_CURRENT_USER, _T("Software\\TeamSoftware Solutions\\PWB")) == ERROR_SUCCESS)
			{
				TCHAR getInfoString[500];
				ULONG lengthVal = 500;
				memset(getInfoString, NULL, 500);
				key.QueryStringValue(_T("App_Location"), getInfoString, &lengthVal);
				// set application string
				szApplication = getInfoString;
			}

			// close key
			key.Close();
		}
		// check string
		if( !szApplication.IsEmpty() )
		{
			// start PWB
			return StartApplication( szApplication, szURL, FALSE, FALSE );
		}
	}
	// navigate to URL
	if( hWnd )
	{
		COPYDATASTRUCT CDS;
		TCHAR sURL[2048];
		// set URL
		memset( sURL, NULL, sizeof( sURL ));
		_tcscpy_s( sURL, 2048, szURL );
		// set navigate type
		DWORD dwType = WM_USER_NAVIGATE_CURRENT;
		if( bOpenNew )
			dwType = WM_USER_NAVIGATE_OPEN;
		else if( m_bCloseSecondary )
			dwType = WM_USER_NAVIGATE_CLOSE;
		// copy data
		CDS.cbData = sizeof( sURL );
		CDS.dwData = dwType;
		CDS.lpData = &sURL;
		// send message to applciation
		::SendMessage( hWnd, WM_COPYDATA, NULL, (LPARAM)(LPVOID) &CDS );
		return TRUE;
	}
	return FALSE;
}

BOOL CPWBNavApp::ReadINIFile(CString szApplication)
{
	// get window handle
	HWND hWnd = ::FindWindow( _T("TSSPWB"), NULL );
	
	// if it is not running start it
	if( !hWnd )
	{
		// get PWB path
		if( szApplication.IsEmpty() )
		{
			CRegKey key;
			// open the registry location and get path
			if( key.Open( HKEY_CURRENT_USER, _T("Software\\TeamSoftware Solutions\\Public Web Browser v2")) == ERROR_SUCCESS )
			{
				TCHAR getInfoString[500];
				ULONG lengthVal = 500;
				memset( getInfoString, NULL, 500 );
				key.QueryStringValue( _T( "App_Location" ), getInfoString, &lengthVal );
				// set application string
				szApplication = getInfoString;
			}
			// close key
			key.Close();

		}
		// check string
		if( !szApplication.IsEmpty() )
		{
			// start PWB
			if( StartApplication( szApplication, NULL, FALSE, TRUE, 2000 ))
			{
				// get window handle
				hWnd = ::FindWindow( _T("TSSPWB"), NULL );
			}
		}
	}
	// navigate to URL
	if( hWnd )
	{
		COPYDATASTRUCT CDS;
		// set read INI type
		DWORD dwType = WM_USER_READ_INI_FILE;
		// copy data
		CDS.cbData = NULL;
		CDS.dwData = dwType;
		CDS.lpData = NULL;
		// send message to applciation
		::SendMessage( hWnd, WM_COPYDATA, NULL, (LPARAM)(LPVOID) &CDS );
		return TRUE;
	}
	return FALSE;
}

BOOL CPWBNavApp::StartApplication(CString szApplication, CString szCommandLine, BOOL bWait, BOOL bIdle, INT nSleep, WORD wShowWindow)
{
	CWaitCursor wait;

	PROCESS_INFORMATION ProcessInfo;
	STARTUPINFO StartupInfo = {0};
	StartupInfo.cb = sizeof(STARTUPINFO);
	StartupInfo.dwFlags = STARTF_USESHOWWINDOW;
	StartupInfo.wShowWindow = wShowWindow;
	// add command line
	szApplication += _T(" ") + szCommandLine;
	// get applciation name
	LPTSTR lpApplicationName = szApplication.GetBuffer( szApplication.GetLength() );
	szApplication.ReleaseBuffer();
	// create process
	if( CreateProcess( NULL, lpApplicationName, NULL, NULL, FALSE, 0, NULL, NULL, &StartupInfo, &ProcessInfo))
	{
		// sleep
		if( nSleep )
			Sleep( nSleep );
		// wait for process to close
		if( bWait )
			WaitForSingleObject( ProcessInfo.hProcess, INFINITE );
		// wait for idle
		else if( bIdle )
			::WaitForInputIdle( ProcessInfo.hProcess, INFINITE );
		// return true
		return TRUE;
	}
    return FALSE;
}
